      DOUBLE PRECISION FUNCTION DAQUAD(RULE,N,F,A,B,TOL,MAXLEV,SING,
     *                                 ERREST, alpha)

C   FOR COMPLETE DOCUMENTATION SEE AQUAD.
C
      DOUBLE PRECISION QRIGHT(30),XSAVE(30),F,DQUAD, alpha,
     *                 A,B,TOL,SING,ERREST,FAC,SUM,CORR,FLAG,
     *                 XL,XR,TOLPU,QPREV,XM,STEP,QLEFT,QNOW,QDIFF,TEMP
      INTEGER LSAVE(30)
      EXTERNAL RULE,F
C
C      ***************
C      *
C      * STRAIGHT INTEGRATION IF MAXLEV = 0.
C      *
C      ***************
C
         IF (MAXLEV .NE. 0) GO TO 4
              DAQUAD = DQUAD(RULE,N,F,A,B, alpha)
      RETURN
C
C      ***************
C      *
C      * OBTAIN  THE  APPROPRIATE SCALING FACTOR  FAC  (FOR THE
C      * CONVERGENCE CRITERION AND THE ERROR ESTIMATE) FROM THE
C      * SUBROUTINE DEFINED BY RULE.
C      *
C      ***************
C
   4     NF = -N
         CALL RULE(NF,A,B,XSAVE,QRIGHT)
         FAC = XSAVE(1)
C
C      ***************
C      *
C      * INITIALIZATION AND CALCULATIONS FOR THE ZERO-TH LEVEL.
C      *
C      ***************
C
   8     SUM = 0.0D0
         CORR = 0.0D0
         ERREST = 0.0D0
         FLAG = 0.0D0
         SING = 0.0D0
         ICOUNT = 0
         XL = A
         XR = B
         TOLPU = TOL*FAC/(B - A)
         QPREV = DQUAD(RULE,N,F,A,B, alpha)

C
C      ***************
C      *
C      * SUCCESSIVELY BISECT THE SUBINTERVALS UNTIL EITHER THE MAXIMUM
C      * ALLOWED LEVEL IS REACHED OR THE INTEGRAL ON A SUBINTERVAL
C      * AGREES WITH THE SUM OF THE INTEGRALS ON ITS 2 HALVES.
C      *
C      ***************
C
         LEV = 1
         LEVEL = 1
  10     XM = 0.5D0*(XL + XR)
         STEP = XR - XL
         QLEFT = DQUAD(RULE,N,F,XL,XM, alpha)
         QRIGHT(LEV) = DQUAD(RULE,N,F,XM,XR, alpha)
         QNOW = QLEFT + QRIGHT(LEV)
         QDIFF = QNOW - QPREV

C
C      ***************
C      *
C      * TEST FOR INTERVAL CONVERGENCE AND CHECK IF THE MAXIMUM LEVEL
C      * IS REACHED.
C      *
C      ***************
C
         IF (DABS(QDIFF) .LE. TOLPU*STEP) GO TO 30
              IF (LEVEL .GE. MAXLEV) GO TO 20
                   XSAVE(LEV) = XR
                   LSAVE(LEV) = LEVEL
                   XR = XM
                   QPREV = QLEFT
                   LEVEL = LEVEL + 1
                   LEV = LEV + 1
                   GO TO 10
C
C      ***************
C      *
C      * MAXIMUM LEVEL WAS REACHED. ACCUMULATE THE TOTAL LENGTH OF THE
C      * SUBINTERVALS ON WHICH THE CONVERGENCE FAILED, AND RECORD THE
C      * LOCATION OF THE SINGULARITY.
C      *
C      ***************
C
  20          FLAG = FLAG + STEP
              SING = XM
C
C      ***************
C      *
C      * THE INTEGRAL ON THE INTERVAL CONVERGED OR THE MAXIMUM ALLOWED
C      * LEVEL WAS REACHED.  ADD CONTRIBUTION INTO THE RUNNING SUMS.
C      *
C      ***************
C
  30     SUM = SUM + QNOW
         CORR = CORR + QDIFF
         ERREST = ERREST + DABS(QDIFF)
         ICOUNT = ICOUNT + 1
         IF (LEV .LE. 1) GO TO 40
              LEV = LEV - 1
              XL = XR
              XR = XSAVE(LEV)
              QPREV = QRIGHT(LEV)
              LEVEL = LSAVE(LEV) + 1
              GO TO 10
C
C      ***************
C      *
C      * ADD THE CORRECTION TERM FROM THE FIRST STEP OF RICHARDSON
C      * EXTRAPOLATION.  MAKE SURE ERREST IS NOT LESS THAN ROUNDOFF
C      * LEVEL.
C      *
C      ***************
C


40     	DAQUAD = SUM + CORR/FAC
         ERREST = ERREST/FAC
         TOL = FLAG
         MAXLEV = ICOUNT*2
         IF (ERREST .EQ. 0.0D0) RETURN
  50     TEMP = DABS(DAQUAD) + ERREST
         IF (TEMP .NE. DABS(DAQUAD)) RETURN
         ERREST = 2.0D0*ERREST
         GO TO 50
      END
